use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
CIDR
DROLSKY
DROLSKY's
Data
Dave
Gregory
IETF
IP
IPv
Neely
Neil
Oschwald
PayPal
Rolsky
Rolsky's
Sonnen
TEREDO
Validate
anycast
autarch
drolsky
goschwald
lib
loopback
multicast
neil
routable
subnet
unicast
